@@ -2,18 +2,14 @@ package Mojo::IOLoop::ForkCall;
 
 use Mojo::Base 'Mojo::EventEmitter';
 
-our $VERSION = '0.060_001';
+our $VERSION = '0.01';
 $VERSION = eval $VERSION;
 
 use Mojo::IOLoop;
-use IO::Pipely 'pipely';
-use POSIX ();
-
-use Perl::OSType 'is_os_type';
-use constant IS_WINDOWS => is_os_type('Windows');
-use constant IS_CYGWIN  => $^O eq 'cygwin';
+use Child;
 
 use Exporter 'import';
+
 our @EXPORT_OK = qw/fork_call/;
 
 has 'ioloop'       => sub { Mojo::IOLoop->singleton };
@@ -23,66 +19,48 @@ has 'weaken'       => 0;
 
 sub run {
   my ($self, $job) = (shift, shift);
-  my ($args, $cb);
-  $args = shift if @_ and ref $_[0] eq 'ARRAY';
-  $cb   = shift if @_;
+  my $args = shift if @_ and ref $_[0] eq 'ARRAY';
+  my $cb   = shift if @_;
 
-  my ($r, $w) = pipely; 
   my $serializer = $self->serializer;
 
-  my $pid = fork;
-  die "Failed to fork: $!" unless defined $pid;
-
-  if ($pid == 0) {
-    # child
-    close $r;
-
+  my $child = $self->_child(sub {
+    my $parent = shift;
     local $@;
     my $res = eval {
       local $SIG{__DIE__};
       $serializer->([undef, $job->(@$args)]);
     };
     $res = $serializer->([$@]) if $@;
-   
-    _send($w, $res);
+    $parent->write($res);
+  });
 
-    # attempt to generalize exiting from child cleanly on all platforms
-    # adapted from POE::Wheel::Run mostly
-    eval { POSIX::_exit(0) } unless IS_WINDOWS;
-    eval { CORE::kill KILL => $$ };
-    exit 0;
+  my $stream = Mojo::IOLoop::Stream->new($child->read_handle);
+  $self->ioloop->stream($stream);
 
-  } else {
-    # parent
-    close $w;
-
-    my $stream = Mojo::IOLoop::Stream->new($r);
-    $self->ioloop->stream($stream);
-
-    my $buffer = '';
-    $stream->on( read  => sub { $buffer .= $_[1] } );
-
-    if ($self->weaken) {
-      require Scalar::Util;
-      Scalar::Util::weaken($self);
-    }
-
-    my $deserializer = $self->deserializer;
-    $stream->on( close => sub {
-      my $res = do {
-        local $@;
-        eval { $deserializer->($buffer) } || [$@];
-      };
-      $self->$cb(@$res) if $cb;
-      $self->emit( finish => @$res ) if $self;
-
-      waitpid $pid, 0;
-    });
-
-    return $pid;
+  my $buffer = '';
+  $stream->on( read  => sub { $buffer .= $_[1] } );
+  if ($self->weaken) {
+    require Scalar::Util;
+    Scalar::Util::weaken($self);
   }
+  $stream->on( close => sub {
+    my $res = do {
+      local $@;
+      eval { $self->deserializer->($buffer) } || [$@];
+    };
+    $self->$cb(@$res) if $cb;
+    $self->emit( finish => @$res ) if $self;
+    return unless $child;
+    # $child->kill(9) unless $child->is_complete; 
+    $child->wait;
+  });
+
+  return $child;
 }
 
+sub _child { Child->new($_[1], pipely => 1)->start }
+
 ## functions
 
 sub fork_call (&@) {
@@ -96,17 +74,6 @@ sub fork_call (&@) {
   });
 }
 
-sub _send {
-  my ($h, $data) = @_;
-  if (IS_WINDOWS || IS_CYGWIN) {
-    my $len = length $data;
-    my $written = 0;
-    $written += syswrite $h, $data, 65536, $written while $written < $len;
-  } else {
-    syswrite $h, $data;
-  }
-}
-
 1;
 
 
@@ -156,7 +123,7 @@ Upon request this module exports the following functions.
 
 =head2 fork_call
 
- fork_call { my @args = @_; child code; return @res } @args, sub { my @res = @_; parent callback }
+ fork_call { my @args = @_; child code; return @res }, @args, sub { my @res = @_; parent callback }
 
 This function is a drop-in replacement for L<AnyEvent::Util>'s C<fork_call>.
 Because it is attempting to mimic that function the api is different to that provided by the OO interface descibed below.
@@ -167,7 +134,7 @@ The callback will receive the deserialized return values from the child block as
 Any error will be available in C<$@>.
 
 The underlying ForkCall object will use the default attributes described below.
-As with C<run> the return value is the child's pid, just in case it is necessary to kill the child process.
+As with C<run> the return value is the L<Child> object, just in case it is needed to kill the child process.
 
 =head1 EVENTS
 
@@ -230,18 +197,18 @@ Takes a code reference (required) which is the job to be run on the child.
 If the next argument is an array reference, these will be passed to the child job.
 If the last argument is a code reference, it will be called immediately before the finish event is emitted, its arguments are the same as the C<finish> event.
 
-Returns the child's pid, just in case you should need to manually kill it.
+Returns the L<Child> object, just in case you should need to manually kill it.
 
 =head1 SEE ALSO
 
 =over
 
-=item L<Mojo::IOLoop>
-
-=item L<AnyEvent::Util>
+=item L<Child>
 
 =item L<IO::Pipely>
 
+=item L<AnyEvent::Util>
+
 =back
 
 =head1 SOURCE REPOSITORY
